==> RcppBlaze_0.1.0.tar.gz: RcppBlaze/inst/include/blaze/util/typetraits/IsConst.h <==
//=================================================================================================
/*!
//  \file blaze/util/typetraits/IsConst.h
//  \brief Header file for the IsConst type trait
//
//  Copyright (C) 2013 Klaus Iglberger - All Rights Reserved
//
//  This file is part of the Blaze library. You can redistribute it and/or modify it under
//  the terms of the New (Revised) BSD License. Redistribution and use in source and binary
//  forms, with or without modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice, this list
//     of conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//  3. Neither the names of the Blaze development group nor the names of its contributors
//     may be used to endorse or promote products derived from this software without specific
//     prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
*/
//=================================================================================================

#ifndef _BLAZE_UTIL_TYPETRAITS_ISCONST_H_
#define _BLAZE_UTIL_TYPETRAITS_ISCONST_H_


//*************************************************************************************************
// Includes
//*************************************************************************************************

#include <boost/type_traits/is_const.hpp>
#include <blaze/util/FalseType.h>
#include <blaze/util/SelectType.h>
#include <blaze/util/TrueType.h>


namespace blaze {

//=================================================================================================
//
//  CLASS DEFINITION
//
//=================================================================================================

//*************************************************************************************************
/*! \cond BLAZE_INTERNAL */
/*!\brief Auxiliary helper struct for the IsConst type trait.
// \ingroup type_traits
*/
template< typename T >
struct IsConstHelper
{
   //**********************************************************************************************
   enum { value = boost::is_const<T>::value };
   typedef typename SelectType<value,TrueType,FalseType>::Type  Type;
   //**********************************************************************************************
};
/*! \endcond */
//*************************************************************************************************


//*************************************************************************************************
/*!\brief Compile time check for constant data types.
// \ingroup type_traits
//
// The IsConst type trait tests whether or not the given template parameter is a (top level)
// const-qualified data type. In case the given data type is const-qualified, the \a value
// member enumeration is set to 1, the nested type definition \a Type is \a TrueType, and
// the class derives from \a TrueType. Otherwise \a value is set to 0, \a Type is \a FalseType,
// and the class derives from \a FalseType.

   \code
   blaze::IsConst<const int>::value          // Evaluates to 1
   blaze::IsConst<const volatile int>::Type  // Results in TrueType
   blaze::IsConst<int* const>                // Is derived from TrueType
   blaze::IsConst<int>::value                // Evaluates to 0
   blaze::IsConst<const int*>::Type          // Results in FalseType (the const-qualifier is not at the top level)
   blaze::IsConst<const int* volatile>       // Is derived from FalseType
   \endcode
*/
template< typename T >
struct IsConst : public IsConstHelper<T>::Type
{
 public:
   //**********************************************************************************************
   /*! \cond BLAZE_INTERNAL */
   enum { value = IsConstHelper<T>::value };
   typedef typename IsConstHelper<T>::Type  Type;
   /*! \endcond */
   //**********************************************************************************************
};
//*************************************************************************************************

} // namespace blaze

#endif

==> RcppBlaze_1.0.2.tar.gz: RcppBlaze/inst/include/blaze/util/typetraits/IsConst.h <==
//=================================================================================================
/*!
//  \file blaze/util/typetraits/IsConst.h
//  \brief Header file for the IsConst type trait
//
//  Copyright (C) 2012-2020 Klaus Iglberger - All Rights Reserved
//
//  This file is part of the Blaze library. You can redistribute it and/or modify it under
//  the terms of the New (Revised) BSD License. Redistribution and use in source and binary
//  forms, with or without modification, are permitted provided that the following conditions
//  are met:
//
//  1. Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice, this list
//     of conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//  3. Neither the names of the Blaze development group nor the names of its contributors
//     may be used to endorse or promote products derived from this software without specific
//     prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//  SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
//  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
//  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
//  DAMAGE.
*/
//=================================================================================================

#ifndef _BLAZE_UTIL_TYPETRAITS_ISCONST_H_
#define _BLAZE_UTIL_TYPETRAITS_ISCONST_H_


//*************************************************************************************************
// Includes
//*************************************************************************************************

#include <type_traits>
#include <blaze/util/IntegralConstant.h>


namespace blaze {

//=================================================================================================
//
//  CLASS DEFINITION
//
//=================================================================================================

//*************************************************************************************************
/*!\brief Compile time check for constant data types.
// \ingroup type_traits
//
// The IsConst type trait tests whether or not the given template parameter is a (top level)
// const-qualified data type. In case the given data type is const-qualified, the \a value
// member constant is set to \a true, the nested type definition \a Type is \a TrueType, and
// the class derives from \a TrueType. Otherwise \a value is set to \a false, \a Type is
// \a FalseType, and the class derives from \a FalseType.

   \code
   blaze::IsConst<const int>::value          // Evaluates to 'true'
   blaze::IsConst<const volatile int>::Type  // Results in TrueType
   blaze::IsConst<int* const>                // Is derived from TrueType
   blaze::IsConst<int>::value                // Evaluates to 'false'
   blaze::IsConst<const int*>::Type          // Results in FalseType (the const-qualifier is not at the top level)
   blaze::IsConst<const int* volatile>       // Is derived from FalseType
   \endcode
*/
template< typename T >
struct IsConst
   : public BoolConstant< std::is_const<T>::value >
{};
//*************************************************************************************************


//*************************************************************************************************
/*!\brief Auxiliary variable template for the IsConst type trait.
// \ingroup type_traits
//
// The IsConst_v variable template provides a convenient shortcut to access the nested \a value
// of the IsConst class template. For instance, given the type \a T the following two statements
// are identical:

   \code
   constexpr bool value1 = blaze::IsConst<T>::value;
   constexpr bool value2 = blaze::IsConst_v<T>;
   \endcode
*/
template< typename T >
constexpr bool IsConst_v = IsConst<T>::value;
//*************************************************************************************************

} // namespace blaze

#endif
